/***************************************************************************/
/***  Name         : log_helpor.sql                                      ***/
/***                                                                     ***/
/***  Description  : Stored procedures used as helpers for logistics     ***/
/***                  modules.                                           ***/
/***                                                                     ***/
/***  Author       : First Choice Software, Inc                          ***/
/***                 4412 Spicewood Springs Road Suite 701               ***/
/***                 Austin, TX 78759                                    ***/
/***                                                                     ***/
/***  Usage        : First Choice Confidential                           ***/
/***                                                                     ***/
/***  Preconditions: None known                                          ***/
/***                                                                     ***/
/***  Postcondition: None known                                          ***/
/***                                                                     ***/
/***  Platforms    : This version supports SQL Server version 6.0 or     ***/
/***                  later.                                             ***/
/***                                                                     ***/
/***                                                                     ***/
/***  Copyright (C) 1997 First Choice Software, Inc.                     ***/
/***  All Rights Reserved                                                ***/
/***                                                                     ***/
/***************************************************************************/

/***************************************************************************/
/* Header declarations.                                                    */
/*                                                                         */
/* Copyright (C) 1998. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

/***************************************************************************/
/* Get item from a Clarify List (gbst_lst).                                */
/*                                                                         */
/* Copyright (C) 1998. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

CREATE OR REPLACE PROCEDURE get_list_item (list_name IN VARCHAR2, 
                                           list_item IN VARCHAR2,
                                           ret_string OUT VARCHAR,
                                           ret_objid OUT NUMBER)

IS
  list_objid NUMBER;
BEGIN
                                        /* Find the list. If not found,    */
                                        /*  error and exit.                */
  BEGIN
    SELECT objid into list_objid from table_gbst_lst 
    WHERE title = list_name;
  EXCEPTION
    WHEN NO_DATA_FOUND
    THEN
      ret_string := '';
      ret_objid := 0;
      goto end_it;
  END;

                                        /* If no item given, find the      */
                                        /*  default. Else make sure that   */
                                        /*  the item is there (and active).*/
  BEGIN
    IF (list_item = '' OR list_item IS NULL)
    THEN
      SELECT title, objid INTO ret_string, ret_objid 
      FROM table_gbst_elm 
      WHERE gbst_elm2gbst_lst = list_objid AND state = 2;
    ELSE
      SELECT title, objid INTO ret_string, ret_objid 
      FROM table_gbst_elm
      WHERE gbst_elm2gbst_lst = list_objid AND title = list_item AND 
            (state = 0 OR state = 2);
    END IF;
  EXCEPTION
    WHEN NO_DATA_FOUND
    THEN
      ret_string := '';
      ret_objid := 0;
      goto end_it;
  END;

<<end_it>>
  NULL;

END;
/

/***************************************************************************/
/* Permission granting statements.                                         */
/*                                                                         */
/* Copyright (C) 1998. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

grant execute on get_list_item to clarify_user;
begin
  dbms_sql.parse(dbms_sql.open_cursor, 'drop public synonym get_list_item', dbms_sql.native);
  exception
  when others
    then null;
end;
/
create public synonym get_list_item for get_list_item;

/***************************************************************************/
/* Test Driver.                                                            */
/*                                                                         */
/* Copyright (C) 1998. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

CREATE OR REPLACE PROCEDURE test_proc
IS
  ret_string VARCHAR2(20);
  ret_objid NUMBER;
BEGIN
  get_list_item('Case Type', 'Questions', ret_string, ret_objid);
  dbms_output.put_line(ret_string);
  dbms_output.put_line(ret_objid);
END test_proc;
/
show errors;

grant execute on test_proc to clarify_user;
begin
  dbms_sql.parse(dbms_sql.open_cursor, 'drop public synonym test_proc', dbms_sql.native);
  exception
  when others
    then null;
end;
/
create public synonym test_proc for test_proc;


