/***************************************************************************/
/***  Name         : transor.sql                                         ***/
/***                                                                     ***/
/***  Description  : Part Transfer Stored Procedures for Oracle.         ***/
/***                                                                     ***/
/***  Author       : First Choice Software, Inc                          ***/
/***                 4412 Spicewood Springs Road Suite 701               ***/
/***                 Austin, TX 78759                                    ***/
/***                                                                     ***/
/***  Usage        : First Choice Confidential                           ***/
/***                                                                     ***/
/***  Preconditions: None known                                          ***/
/***                                                                     ***/
/***  Postcondition: None known                                          ***/
/***                                                                     ***/
/***  Platforms    : This version supports Oracle version 7.1 or         ***/
/***                  later.                                             ***/
/***                                                                     ***/
/***                                                                     ***/
/***  Copyright (C) 1999 First Choice Software, Inc.                     ***/
/***  All Rights Reserved                                                ***/
/***                                                                     ***/
/***************************************************************************/

/***************************************************************************/
/* Transfer part(s).                                                       */
/*                                                                         */
/* Copyright (C) 1999. First Choice Software. All Rights Reserved.         */
/***************************************************************************/

CREATE OR REPLACE PROCEDURE trans_internal(pr_objid IN NUMBER, 
                                           other_objid IN NUMBER,
                                           mod_objid IN NUMBER,
                                           part_objid IN NUMBER,
                                           f_bin_objid IN NUMBER,
                                           f_loc_objid IN NUMBER,
                                           from_good IN NUMBER,
                                           t_bin_objid IN NUMBER,
                                           t_loc_objid IN NUMBER,
                                           to_good IN NUMBER,
                                           quantity IN NUMBER,
                                           serial_num IN VARCHAR2,
                                           trans_type IN NUMBER,
                                           user_objid IN NUMBER,
                                           trans_date IN VARCHAR2,
                                           ref_id IN VARCHAR2,
                                           notes IN VARCHAR2,
                                           gen_act IN NUMBER,
                                           use_fifo IN NUMBER,
                                           update_cost IN VARCHAR2,
                                           trans_objid OUT NUMBER,
                                           ret_val OUT NUMBER)
IS                                          
BEGIN
  ret_val := 0;
END;
/
show errors;

/***************************************************************************/
/* Transfer part(s).                                                       */
/*                                                                         */
/* Copyright (C) 1999. First Choice Software. All Rights Reserved.         */
/***************************************************************************/

CREATE OR REPLACE PROCEDURE part_transfer(
		  from_loc IN VARCHAR2, 
		  from_bin IN VARCHAR2,
		  from_good IN NUMBER,
		  to_loc IN VARCHAR2,
		  to_bin IN VARCHAR2,
		  to_good IN NUMBER,
		  part_num IN VARCHAR2,
		  mod_levell IN VARCHAR2,
		  domain IN VARCHAR2,
		  quantity IN NUMBER,
                  serial_num IN VARCHAR2,
		  user_name IN VARCHAR2,
		  ref_id IN VARCHAR2,
		  notes IN VARCHAR2,
                  trans_date IN VARCHAR2,
		  wo_num IN VARCHAR2,   /* Not currently used.             */
		  use_fifo IN NUMBER,
		  update_cost IN VARCHAR2,
                  gen_tb IN NUMBER,
		  ret_val IN OUT NUMBER)
IS
  t_date DATE;                          /* Transaction date in date format.*/
  past_time DATE;                       /* A time in the past for time_bomb*/
  sn_track NUMBER;                      /* Is it a serialized part?        */
  part_objid NUMBER;                    /* Part number objid.              */
  mod_objid NUMBER;                     /* Part revision objid.            */
  f_loc_objid NUMBER;                   /* From location objid.            */
  f_bin_objid NUMBER;                   /* From bin objid.                 */
  t_loc_objid NUMBER;                   /* To location objid.              */
  t_bin_objid NUMBER;                   /* To bin objid.                   */
  user_objid NUMBER;                    /* Objid of user record.           */
  trans_objid NUMBER;                   /* Part transaction objid created. */
  user_name1 VARCHAR(30);               /* User name string.               */
BEGIN
					/* Get current date/time.          */
					/*  Only if one isn't supplied.    */
                                        /* Do so for time in the past.     */
  BEGIN
    IF LTRIM(RTRIM(trans_date)) = '' OR LTRIM(RTRIM(trans_date)) IS NULL
    THEN
      t_date := SYSDATE;
    ELSE
      IF INSTR(trans_date, ' ') > 0
      THEN
         t_date := TO_DATE(trans_date, 'MM/DD/RR HH:MI:SS AM');
      ELSE
         t_date := TO_DATE(trans_date, 'MM/DD/RR');
      END IF;
    END IF;
    past_time := ADD_MONTHS(t_date, -1);
  END;
  
					/* If no user name specified, find */
					/*  one in the config item. If that*/
					/*  isn't found, use default.      */
  BEGIN
    IF user_name IS NULL
    THEN
      SELECT str_value INTO user_name1 FROM table_config_itm 
      WHERE name = 'Default Logistics User';
    ELSE
      user_name1 := user_name;
    END IF;
  EXCEPTION
    WHEN NO_DATA_FOUND
    THEN
      user_name1 := 'ClearLogistics';
  END;
  
                                        /* Can't transfer from somewhere   */
                                        /*  to itself.                     */
  IF from_bin = to_bin AND from_loc = to_loc AND from_good = to_good
  THEN
     ret_val := -1;
     GOTO end_proc;
  END IF;

                                        /* Can't transfer < 1 units.       */
  IF quantity < 1
  THEN
     ret_val := -2;
     GOTO end_proc;
  END IF;

                                        /* If quantity > 1, can't have a   */
                                        /*  serial number.                 */
  IF quantity > 1 AND serial_num IS NOT NULL
  THEN
     ret_val := -3;
     GOTO end_proc;
  END IF;

                                        /* Get SN Track from part number.  */
                                        /* If part not found, error/exit.  */
  BEGIN
    SELECT sn_track, objid INTO sn_track, part_objid
      FROM table_part_num 
      WHERE part_number = part_num AND domain = domain;
    EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
        ret_val := -4;
        GOTO end_proc;
  END;

                                        /* If the part is serialized, and  */
                                        /*  no serial number, error.       */
  IF sn_track = 1 AND serial_num IS NULL
  THEN
     ret_val := -5;
     GOTO end_proc;
  END IF;

                                        /* Cannot find the part revision.  */
  BEGIN
    SELECT objid INTO mod_objid FROM table_mod_level
      WHERE ((mod_level IS NULL AND mod_levell IS NULL) OR (mod_level = mod_levell))
            AND part_info2part_num = part_objid;
    EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
        ret_val := -6;
        GOTO end_proc;
  END;
  
                                        /* Make sure from location is OK.  */
  BEGIN
    SELECT objid INTO f_loc_objid FROM table_inv_locatn
      WHERE location_name = from_loc AND active = 1 AND inv_class <> 1;
    EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
        ret_val := -7;
        GOTO end_proc;
  END;
  
                                        /* Make sure from bin is OK.       */
  BEGIN
    IF from_bin IS NULL
    THEN
      SELECT objid INTO f_bin_objid FROM table_inv_bin
        WHERE bin_name IS NULL AND active = 1 AND 
              inv_bin2inv_locatn = f_loc_objid;
    ELSE
      SELECT objid INTO f_bin_objid FROM table_inv_bin
        WHERE bin_name = from_bin AND active = 1 AND 
              inv_bin2inv_locatn = f_loc_objid;
    END IF;
    EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
        ret_val := -8;
        GOTO end_proc;
  END;

                                        /* Make sure to location is OK.    */
  BEGIN
    SELECT objid INTO t_loc_objid FROM table_inv_locatn
      WHERE location_name = to_loc AND active = 1 AND inv_class <> 1;
    EXCEPTION
      WHEN NO_DATA_FOUND
    THEN
      ret_val := -9;
      GOTO end_proc;
  END;

                                        /* Make sure to bin is OK.         */
  BEGIN
    IF to_bin IS NULL
    THEN
      SELECT objid INTO t_bin_objid FROM table_inv_bin
        WHERE bin_name IS NULL AND active = 1 AND 
              inv_bin2inv_locatn = t_loc_objid;
    ELSE
      SELECT objid INTO t_bin_objid FROM table_inv_bin
        WHERE bin_name = to_bin AND active = 1 AND 
              inv_bin2inv_locatn = t_loc_objid;
    END IF;
    EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
        ret_val := -10;
        GOTO end_proc;
  END;

                                        /* Make sure that user is OK.      */
  BEGIN
    SELECT objid INTO user_objid FROM table_user WHERE login_name = user_name1;
    EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
        ret_val := -11;
        GOTO end_proc;
  END;

                                        /* Call on internal version of part*/
                                        /*  transfer. If error, subtract   */
                                        /*  100 to error number. Exit.     */  
  trans_internal(0, 0, mod_objid, part_objid, f_bin_objid, f_loc_objid,
                      from_good, t_bin_objid, t_loc_objid, to_good, quantity,
                      serial_num, 4, user_objid, t_date, ref_id, notes,
                      1, use_fifo, update_cost, trans_objid, ret_val);
  IF ret_val < 0
  THEN
     ret_val := ret_val - 100;
     GOTO end_proc;
  END IF;

  ret_val := 0;
<<end_proc>> NULL;
END;
/
show errors;

/***************************************************************************/
/* Transfer part(s).                                                       */
/*                                                                         */
/* Copyright (C) 1999. First Choice Software. All Rights Reserved.         */
/***************************************************************************/

CREATE OR REPLACE PROCEDURE trans_internal(pr_objid IN NUMBER, 
                                           other_objid IN NUMBER,
                                           mod_objid IN NUMBER,
                                           part_objid IN NUMBER,
                                           f_bin_objid IN NUMBER,
                                           f_loc_objid IN NUMBER,
                                           from_good IN NUMBER,
                                           t_bin_objid IN NUMBER,
                                           t_loc_objid IN NUMBER,
                                           to_good IN NUMBER,
                                           quantity IN NUMBER,
                                           serial_num IN VARCHAR2,
                                           trans_type IN NUMBER,
                                           user_objid IN NUMBER,
                                           trans_date IN VARCHAR2,
                                           ref_id IN VARCHAR2,
                                           notes IN VARCHAR2,
                                           gen_act IN NUMBER,
                                           use_fifo IN NUMBER,
                                           update_cost IN VARCHAR2,
                                           trans_objid IN OUT NUMBER,
                                           ret_val IN OUT NUMBER)
IS
  is_serial NUMBER;                     /* Is it a serialized transaction. */
  move_type NUMBER;                     /* Part movement type (0-3).       */
  cost_decimal DECIMAL(19,4);           /* Cost of transfer (as a decimal  */
                                        /*  for Clarify 6.0+).             */
  cost_real FLOAT;                      /* Cost for transfer (as a real for*/
                                        /*  Clarify before 6.0).           */
  schema_rev NUMBER;                    /* Schema revision of the DB.      */
  mod_level VARCHAR2(10);               /* Revision (mod_level).           */
  part_num VARCHAR2(30);                /* Part number.                    */
  trans_id VARCHAR2(80);                /* Transaction ID.                 */
  pgq_from NUMBER; 
  pbq_from NUMBER;                      /* Part good/bad qty for from inst.*/
  pgq_to NUMBER; 
  pbq_to NUMBER;                        /* Part good/bad qty for to inst.  */
  bin_id NUMBER;                        /* Bin objid of part_view record.  */
  t_bin_name VARCHAR2(20);              /* Name of the "to" bin.           */
  f_bin_name VARCHAR2(20);              /* Name of the "from" bin.         */
  t_loc_name VARCHAR2(20);              /* Name of the "to" location.      */
  f_loc_name VARCHAR2(20);              /* Name of the "from" location.    */
  add_str VARCHAR2(255);                /* Additional info string.         */
  from_class NUMBER;                    /* Inv class of from location.     */
  to_class NUMBER;                      /* Inv class of to location.       */
  num_fifo NUMBER;                      /* Number of FIFO units avail.     */
  direction VARCHAR2(10);               /* Direction of transfer.          */
  left_to_cost NUMBER;                  /* Number of units left to cost.   */
  total_cost FLOAT;                     /* Amount of costing.              */
  total_cost_dec DECIMAL(19,4);         /* Amount of costing (6.0+).       */
  qty_avail NUMBER;                     /* Number of FIFO objects at a cost*/
  cost FLOAT;                           /* FIFO cost.                      */
  date_avail DATE;                      /* Date of FIFO transaction.       */
  gbst_objid NUMBER;                    /* Activity string objid.          */
  base_objid NUMBER;                    /* Base objid for all sequences.   */
  from_inst_objid NUMBER;               /* Objid of from part inst record. */
  to_inst_objid NUMBER;                 /* Objid of to part inst record.   */
  dtl_objid NUMBER;                     /* Part request objid if needed.   */
  act_objid NUMBER;                     /* Activity log objid.             */
  o_objid NUMBER;                       /* Other objid for receive parts.  */
  f_auth_objid NUMBER;                  /* From part auth level.           */
  t_auth_objid NUMBER;                  /* To part auth level.             */
  loc_objid NUMBER;                     /* Location FIFO is coming from.   */
  fifo_objid NUMBER;                    /* Objid of FIFO record.           */
  new_fifo_objid NUMBER;                /* Objid of new built FIFO record. */
  new_hdr_int NUMBER;                   /* Quantity-tracked hdr_ind flag.  */
  t_str VARCHAR2(255);                  /* Temporary string.               */

  CURSOR fifo_cursor1 IS 
        SELECT objid, qty_avail, cost, date_avail
          FROM table_fifo_cost
          WHERE fifo_cost2inv_locatn = loc_objid AND fifo_cost2part_number = part_objid
          ORDER BY date_avail;

  CURSOR fifo_cursor2 IS
        SELECT objid, qty_avail, cost, date_avail
          FROM table_fifo_cost
          WHERE fifo_cost2inv_locatn = loc_objid AND fifo_cost2part_number = part_objid
          ORDER BY date_avail desc;
BEGIN
                                        /* Set serialized flag.            */
  IF serial_num IS NOT NULL
  THEN
     is_serial := 1;
  ELSE
     is_serial := 0;
  END IF;

                                        /* Get the activity string objid.  */
                                        /* If not found, error and leave.  */
  BEGIN
    SELECT objid INTO gbst_objid FROM table_gbst_elm WHERE rank = 12300;
  EXCEPTION    
    WHEN NO_DATA_FOUND
    THEN
       ret_val := -22;
       GOTO end_proc;
  END;

					/* Get base objid for database.    */
  SELECT site_id*(268435456) INTO base_objid FROM adp_db_header;

                                        /* Set the movement type.          */
  move_type := 0;
  IF from_good = 0
  THEN
     move_type := 2;
  END IF;
  IF to_good = 0
  THEN
     move_type := move_type + 1;
  END IF;

                                        /* Get the revision level of DB.   */
  SELECT schema_rev INTO schema_rev FROM adp_db_header;

                                        /* Get some needed data.           */
  SELECT mod_level INTO mod_level FROM table_mod_level WHERE objid = mod_objid;
  SELECT part_number INTO part_num FROM table_part_num WHERE objid = part_objid;
  SELECT bin_name INTO t_bin_name FROM table_inv_bin WHERE objid = t_bin_objid;
  SELECT bin_name INTO f_bin_name FROM table_inv_bin WHERE objid = f_bin_objid;
  SELECT location_name, inv_class INTO t_loc_name, to_class
    FROM table_inv_locatn WHERE objid = t_loc_objid;
  SELECT location_name, inv_class INTO f_loc_name, from_class
    FROM table_inv_locatn WHERE objid = f_loc_objid;

                                        /* Figure out direction: IN, OUT   */
                                        /*  or TRAN. Tran = transfer among */
                                        /*  inv_locations. IN is from an   */
                                        /*  expense GL and OUT is to an exp*/
  IF from_class <> 2
  THEN
     loc_objid := f_loc_objid;
     IF to_class = 2
     THEN
        direction := 'OUT';
     ELSE
        direction := 'TRAN';
     END IF;
  ELSE 
     loc_objid := t_loc_objid;
     direction := 'IN';
  END IF;

                                        /* If they don't use FIFO cost,    */
                                        /*  then just find costs from      */
                                        /*  normal means.                  */
  IF use_fifo = 0
  THEN
     GOTO find_normal_cost;
  END IF;

                                        /* If they supply the unit cost... */
                                        /* (Work order only for now).      */
                                        /* Build up the proper cost.       */
  IF use_fifo = 2
  THEN
     IF schema_rev >= 45
     THEN
        cost_decimal := TO_NUMBER(update_cost, '999999999999999.99999') * quantity;
     ELSE
        cost_real := TO_NUMBER(update_cost) * quantity;
     END IF;                                  
     GOTO check_auths;
  END IF;

                                        /* If we're using FIFO...          */
                                        /* Find out how many units avail.  */
                                        /* If none (or not enough), error. */
  SELECT sum(qty_avail) INTO num_fifo FROM table_fifo_cost
    WHERE fifo_cost2inv_locatn = loc_objid AND 
	  fifo_cost2part_number = part_objid;
  IF num_fifo IS NULL OR (num_fifo < quantity AND direction <> 'IN')
  THEN
     ret_val := -24;
     GOTO end_proc;
  END IF;
     

                                        /* Remember how many units they    */
                                        /*  are moving. Start with no cost.*/
					/* While there's more to move...   */
  left_to_cost := quantity;
  total_cost := 0;
  total_cost_dec := 0;

  LOOP
    EXIT WHEN left_to_cost = 0;

                                        /* Now we need a cursor to look at */
                                        /*  the fifo_cost records in order */
                                        /*  and to consume the units as    */
                                        /*  needed.                        */
                                        /* For out or transfer, use FIFO.  */
                                        /* If coming in, we're adding to   */
                                        /*  the most recent cost (LIFO).   */
    IF direction <> 'IN'
    THEN
      OPEN fifo_cursor1;
    ELSE
      OPEN fifo_cursor2;
    END IF;

					/* Open the cursor. Get the next   */
					/*  fifo cost.                     */
					/* If inbound, add qty to the fifo */
					/*  cost record and transact all.  */
                                        /* Get total cost for transaction. */
    BEGIN
      LOOP
        IF direction <> 'IN'
        THEN
           FETCH fifo_cursor1 INTO fifo_objid, qty_avail, cost, date_avail;
           EXIT WHEN fifo_cursor1%NOTFOUND;
        ELSE
           FETCH fifo_cursor2 INTO fifo_objid, qty_avail, cost, date_avail;
           EXIT WHEN fifo_cursor2%NOTFOUND;
        END IF;

        IF direction = 'IN'
        THEN
  	   UPDATE table_fifo_cost SET qty_avail = qty_avail + left_to_cost
	     WHERE objid = fifo_objid;
           total_cost := cost * quantity;
           left_to_cost := 0;
	   GOTO end_cursor;
        END IF;        

					/* If there's enough in this fifo  */
					/*  record to handle it all, update*/
					/*  the fifo cost record and trans */
					/*  act all of it.                 */
					/* Else, delete the fifo cost rec  */
					/*  and only transact that amount. */
                                        /* Update total cost for these the */
                                        /*  parts being used.              */
        IF qty_avail > left_to_cost
        THEN
           IF f_loc_objid <> t_loc_objid
           THEN 
	      UPDATE table_fifo_cost SET qty_avail = qty_avail - left_to_cost
	        WHERE objid = fifo_objid;
	   END IF;
           total_cost := total_cost + cost * left_to_cost;
           qty_avail := left_to_cost;
           left_to_cost := 0;
        ELSE
           IF f_loc_objid <> t_loc_objid
           THEN
	      DELETE FROM table_fifo_cost WHERE objid = fifo_objid          ;
	   END IF;
           total_cost := total_cost + cost * qty_avail;
	   left_to_cost := left_to_cost - qty_avail;
        END IF;

					/* If this is a transfer, build    */
					/*  a new fifo cost rec for the    */
					/*  inventory moved in.            */
        IF direction = 'TRAN' AND f_loc_objid <> t_loc_objid
        THEN
           SELECT seq_fifo_cost.nextval INTO new_fifo_objid FROM dual;
  	   new_fifo_objid := new_fifo_objid + base_objid;
	   INSERT INTO table_fifo_cost (objid, date_avail, qty_avail, cost, 
	  	  fifo_cost2part_number, fifo_cost2inv_locatn) VALUES
		  (new_fifo_objid, date_avail, qty_avail, cost,
		   part_objid, t_loc_objid);
        END IF;

        IF left_to_cost = 0
        THEN
           GOTO end_cursor;
        END IF;
      END LOOP;
    END;

<<end_cursor>>
    IF direction <> 'IN'
    THEN
       CLOSE fifo_cursor1;
    ELSE
       CLOSE fifo_cursor2;
    END IF;
  END LOOP;

                                        /* Change to the correct format    */
                                        /*  based on Clarify version.      */
                                        /* Don't need to figure out the    */
                                        /*  standard cost.                 */
  IF schema_rev >= 45
  THEN
     cost_decimal := TO_NUMBER(total_cost, '999999999999999.99999');
  ELSE
     cost_real := TO_NUMBER(total_cost);
  END IF;                                  
  GOTO check_auths;

                                        /* Get the price of the part/rev.  */
                                        /* If no std cost record, price=0. */
                                        /* We need to get it one of two    */
                                        /*  ways (decimal/float).          */
<<find_normal_cost>>
  BEGIN
  IF schema_rev >= 45
  THEN
     SELECT price INTO cost_decimal FROM table_price_num
       WHERE type = 0 AND part_mod = mod_objid AND eff_date <= trans_date
             AND exp_date >= trans_date;
     cost_decimal := cost_decimal * quantity;
  ELSE
     SELECT price INTO cost_real FROM table_price_num
       WHERE type = 0 AND part_mod = mod_objid AND eff_date <= trans_date
             AND exp_date >= trans_date;
     cost_real := cost_real * quantity;
  END IF;
  EXCEPTION
  WHEN NO_DATA_FOUND
  THEN
    cost_decimal := 0;
    cost_real := 0;
  END;

<<check_auths>>

                                        /* Get the objids for the part auth*/
                                        /*  records in case we need to go  */
                                        /*  later and modify them (and the */
                                        /*  auth levels.                   */
  BEGIN
    SELECT objid INTO f_auth_objid FROM table_part_auth 
      WHERE part_auth2inv_bin = f_bin_objid AND n_auth_parts2part_mod = mod_objid;
  EXCEPTION
  WHEN NO_DATA_FOUND
  THEN NULL;
  END;
  BEGIN
    SELECT objid INTO t_auth_objid FROM table_part_auth 
      WHERE part_auth2inv_bin = t_bin_objid AND n_auth_parts2part_mod = mod_objid;
  EXCEPTION
  WHEN NO_DATA_FOUND
  THEN NULL;
  END;

                                        /* First, set a relation if needed.*/
  IF trans_type = 2
  THEN
     o_objid := other_objid;
     IF pr_objid IS NOT NULL
     THEN
        dtl_objid := pr_objid;
     ELSE
        dtl_objid := NULL;
     END IF;
  ELSE
     o_objid := NULL;
  END IF;

                                        /* If serialized, get the one price*/
                                        /*  instance. If not found, build  */
                                        /*  it.                            */
  IF is_serial = 1
  THEN
    BEGIN
      SELECT objid, bin_id INTO from_inst_objid, bin_id FROM table_parts_view
        WHERE part_num_id = mod_objid AND part_serial_no = serial_num;

                                        /* Else, the part_inst already     */
                                        /*  exists. Make sure it's in the  */
                                        /*  proper bin already. If not, it */
                                        /*  is an error. Leave.            */
       IF bin_id <> f_bin_objid
       THEN
          ret_val := -20;
          GOTO end_proc;
       END IF;

      EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
       SELECT seq_part_inst.nextval INTO from_inst_objid FROM dual;
       from_inst_objid := from_inst_objid + base_objid;
       INSERT INTO table_part_inst (objid, part_good_qty, part_bad_qty, 
               part_serial_no, part_mod, part_bin, last_pi_date, pi_tag_no, 
               last_cycle_ct, next_cycle_ct, last_mod_time,  
               last_trans_time, transaction_id, date_in_serv, warr_end_date, 
               repair_date, part_status, pick_request, good_res_qty, 
               bad_res_qty, n_part_inst2part_mod) VALUES (from_inst_objid, 
               0, 0, serial_num, mod_level, '', '', '', '', '', '', '', '',
               trans_date, '', '', '', '', 0, 0, mod_objid);
    END;

                                        /* Build the part transaction ID.  */
                                        /* If a problem, error and leave.  */
    build_id_number('Part Transaction ID', trans_id, ret_val);
    IF ret_val < 0
    THEN
       ret_val := -23;
       GOTO end_proc;
    END IF;

                                        /* Set the part good/bad quantities*/
    IF to_good = 1
    THEN
       pgq_from := 1;
       pbq_from := 0;
    ELSE
       pgq_from := 0;
       pbq_from := 1;
    END IF;

                                        /* Get necessary objids.           */
    SELECT seq_part_trans.nextval INTO trans_objid FROM dual;
    trans_objid := trans_objid + base_objid;
    IF gen_act = 1
    THEN
       SELECT seq_act_entry.nextval INTO act_objid FROM dual;
       act_objid := act_objid + base_objid;
       t_str := TO_CHAR(quantity);
       add_str := 'Reference: ' || ref_id || ', From Location: ' ||
              f_loc_name || ', From Bin: ' || f_bin_name || ', To Location: ' ||
              t_loc_name || ', To Bin: ' || t_bin_name || ', Qty: ' || t_str ||
              ', Comments: ' || notes;
    END IF;

                                        /* Begin transaction.              */
    BEGIN
                                        /* Update the part_inst record.    */
      UPDATE table_part_inst SET part_good_qty = pgq_from, part_bad_qty = pbq_from,
               part_bin = t_bin_name, last_mod_time = trans_date, 
               last_trans_time = trans_date,
               transaction_id = trans_id, part_inst2inv_bin = t_bin_objid
        WHERE objid = from_inst_objid;
      IF dtl_objid IS NOT NULL
      THEN
         UPDATE table_demand_dtl SET demand_dtl2part_inst = from_inst_objid
           WHERE objid = dtl_objid;
      END IF;

                                        /* Now write the part_trans record.*/
                                        /* Two versions, for float/decimal.*/
     IF schema_rev >= 45
     THEN
        INSERT INTO table_part_trans (objid, transaction_id, creation_date, 
                reference_no, part_number, quantity, movement_type, notes,
                standard_cost, trans_type, ROL_ind, open_ord_ind, arch_ind,
                part_trans_owner2user, from_bin2inv_bin, to_bin2inv_bin,
                from_inst2part_inst, to_inst2part_inst, part_trans2demand_dtl, 
                recv_trans2recv_parts) VALUES
                (trans_objid, trans_id, trans_date, ref_id, part_num, 1,
                move_type, notes, cost_decimal, trans_type, 0, 0, 0, 
                user_objid, f_bin_objid, t_bin_objid, from_inst_objid, 
                from_inst_objid, pr_objid, o_objid);
     ELSE
        INSERT INTO table_part_trans (objid, transaction_id, creation_date, 
                reference_no, part_number, quantity, movement_type, notes,
                standard_cost, trans_type, ROL_ind, open_ord_ind, arch_ind,
                part_trans_owner2user, from_bin2inv_bin, to_bin2inv_bin,
                from_inst2part_inst, to_inst2part_inst, part_trans2demand_dtl, 
                recv_trans2recv_parts) VALUES
                (trans_objid, trans_id, trans_date, ref_id, part_num, 1,
                move_type, notes, cost_real, trans_type, 0, 0, 0, 
                user_objid, f_bin_objid, t_bin_objid, from_inst_objid, 
                from_inst_objid, pr_objid, o_objid);
     END IF;
    END;
  ELSE
                                        /* Get the from part inst.         */
                                        /* If not there, and is an inv loc */
                                        /*  that's an error. Leave.        */
                                        /* If not there and is EXP GL,     */
                                        /*  build it. If it is there, make */
                                        /*  sure we're not making the quant*/
                                        /*  go negative. If so, error/exit.*/
    BEGIN
      SELECT objid, part_good_qty, part_bad_qty INTO from_inst_objid, pgq_from, pbq_from
        FROM table_parts_view
        WHERE part_num_id = mod_objid AND bin_id = f_bin_objid;

      IF from_class = 0
      THEN
        IF (from_good = 1 AND quantity > pgq_from) OR 
           (from_good = 0 AND quantity > pbq_from)
        THEN
           ret_val := -21;
           GOTO end_proc;
        END IF;
      END IF;

      EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
       IF from_class = 0
       THEN
          ret_val := -21;
          GOTO end_proc;
       ELSE
          SELECT seq_part_inst.nextval INTO from_inst_objid FROM dual;
          from_inst_objid := from_inst_objid + base_objid;
          INSERT INTO table_part_inst (objid, part_good_qty, part_bad_qty, 
                  part_serial_no, part_mod, part_bin, last_pi_date, pi_tag_no, 
                  last_cycle_ct, next_cycle_ct, last_mod_time,  
                  last_trans_time, transaction_id, date_in_serv, warr_end_date, 
                  repair_date, part_status, pick_request, good_res_qty, 
                  bad_res_qty, n_part_inst2part_mod, part_inst2inv_bin) 
                  VALUES (from_inst_objid, 
                  0, 0, '', mod_level, f_bin_name, '', '', '', '', '', '', '',
                  trans_date, '', '', '', '', 0, 0, mod_objid, f_bin_objid);
          pgq_from := 0;
          pbq_from := 0;
       END IF;
    END;

                                        /* If there's no "to" part_inst... */
                                        /*  then build it.                 */
    BEGIN
      SELECT objid, part_good_qty, part_bad_qty INTO to_inst_objid, pgq_to, pbq_to
        FROM table_parts_view
        WHERE part_num_id = mod_objid AND bin_id = t_bin_objid;
      EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
       SELECT seq_part_inst.nextval INTO to_inst_objid FROM dual;
       to_inst_objid := to_inst_objid + base_objid;
       INSERT INTO table_part_inst (objid, part_good_qty, part_bad_qty, 
               part_serial_no, part_mod, part_bin, last_pi_date, pi_tag_no, 
               last_cycle_ct, next_cycle_ct, last_mod_time,  
               last_trans_time, transaction_id, date_in_serv, warr_end_date, 
               repair_date, part_status, pick_request, good_res_qty, 
               bad_res_qty, n_part_inst2part_mod, part_inst2inv_bin) 
               VALUES (to_inst_objid, 
               0, 0, '', mod_level, t_bin_name, '', '', '', '', '', '', '',
               trans_date, '', '', '', '', 0, 0, mod_objid, t_bin_objid);
       pgq_to := 0;
       pbq_to := 0;    
    END;

                                        /* Build the part transaction ID.  */
                                        /* If a problem, error and leave.  */
    build_id_number('Part Transaction ID', trans_id, ret_val);
    IF ret_val < 0
    THEN
       ret_val := -23;
       GOTO end_proc;
    END IF;

                                        /* Adjust totals from transaction. */
    IF from_good = 1
    THEN
       pgq_from := pgq_from - quantity;
    ELSE
       pbq_from := pbq_from - quantity;
    END IF;
    IF to_good = 1
    THEN
       pgq_to := pgq_to + quantity;
    ELSE
       pbq_to := pbq_to + quantity;
    END IF;

                                        /* Get necessary objids.           */
    SELECT seq_part_trans.nextval INTO trans_objid FROM dual;
    trans_objid := trans_objid + base_objid;
    IF gen_act = 1
    THEN
       SELECT seq_act_entry.nextval INTO act_objid FROM dual;
       act_objid := act_objid + base_objid;
       t_str := TO_CHAR(quantity);
       add_str := 'Reference: ' || ref_id || ', From Location: ' ||
              f_loc_name || ', From Bin: ' || f_bin_name || ', To Location: ' ||
              t_loc_name || ', To Bin: ' || t_bin_name || ', Qty: ' || t_str ||
              ', Comments: ' || notes;
    END IF;

    IF pbq_from = 0 And pgq_from = 0
    THEN
       new_hdr_ind := 3;
    ElSE
       new_hdr_ind := 2;
    END IF;

                                        /* Start the transaction.          */
                                        /* Update the from inst record.    */
                                        /* Update the to inst record.      */
    BEGIN
      UPDATE table_part_inst SET part_good_qty = pgq_from, 
             part_bad_qty = pbq_from, last_mod_time = trans_date,
             last_trans_time = trans_date, transaction_id = trans_id,
             hdr_ind = new_hdr_ind
        WHERE objid = from_inst_objid;
      UPDATE table_part_inst SET part_good_qty = pgq_to, 
             part_bad_qty = pbq_to, last_mod_time = trans_date,
             last_trans_time = trans_date, transaction_id = trans_id,
             hdr_ind = 2
        WHERE objid = to_inst_objid;
      IF dtl_objid IS NOT NULL
      THEN
         UPDATE table_demand_dtl SET demand_dtl2part_inst = from_inst_objid
           WHERE objid = dtl_objid;
      END IF;

                                        /* Now write the part_trans record.*/
                                        /* Two versions, for float/decimal.*/
      IF schema_rev >= 45
      THEN
         INSERT INTO table_part_trans (objid, transaction_id, creation_date, 
                 reference_no, part_number, quantity, movement_type, notes,
                 standard_cost, trans_type, ROL_ind, open_ord_ind, arch_ind,
                 part_trans_owner2user, from_bin2inv_bin, to_bin2inv_bin,
                 from_inst2part_inst, to_inst2part_inst, part_trans2demand_dtl, 
                 recv_trans2recv_parts) VALUES
                 (trans_objid, trans_id, trans_date, ref_id, part_num, quantity,
                 move_type, notes, cost_decimal, trans_type, 0, 0, 0, 
                 user_objid, f_bin_objid, t_bin_objid, from_inst_objid, 
                 to_inst_objid, pr_objid, o_objid);
      ELSE
         INSERT INTO table_part_trans (objid, transaction_id, creation_date, 
                 reference_no, part_number, quantity, movement_type, notes,
                 standard_cost, trans_type, ROL_ind, open_ord_ind, arch_ind,
                 part_trans_owner2user, from_bin2inv_bin, to_bin2inv_bin,
                 from_inst2part_inst, to_inst2part_inst, part_trans2demand_dtl, 
                 recv_trans2recv_parts) VALUES
                 (trans_objid, trans_id, trans_date, ref_id, part_num, quantity,
                 move_type, notes, cost_real, trans_type, 0, 0, 0, 
                 user_objid, f_bin_objid, t_bin_objid, from_inst_objid, 
                 to_inst_objid, pr_objid, o_objid);
      END IF;
    END;
  END IF;
  
                                        /* Now we must do the part auth.   */
                                        /* Do it for from/to and good/bad  */
                                        /*  parts qty.                     */
  IF f_auth_objid IS NOT NULL
  THEN
     IF from_good = 1
     THEN
        UPDATE table_part_auth SET part_good_qoh = part_good_qoh - quantity
          WHERE objid = f_auth_objid;
     ELSE
        UPDATE table_part_auth SET part_bad_qoh = part_bad_qoh - quantity
          WHERE objid = f_auth_objid;
     END IF;
  END IF;
  IF t_auth_objid IS NOT NULL
  THEN
     IF to_good = 1
     THEN
       UPDATE table_part_auth SET part_good_qoh = part_good_qoh + quantity
          WHERE objid = t_auth_objid;
     ELSE
        UPDATE table_part_auth SET part_bad_qoh = part_bad_qoh + quantity
          WHERE objid = t_auth_objid;
     END IF;
  END IF;

                                        /* If they wanted an act_entry...  */
                                        /*  build it!!                     */
  IF gen_act = 1
  THEN
     INSERT INTO table_act_entry (objid, act_code, entry_time, addnl_info, proxy,
       removed, entry_name2gbst_elm, act_entry2user, act_entry2part_trans) 
       VALUES (act_objid, 12300, trans_date, add_str, '', 0, gbst_objid, 
               user_objid, trans_objid);
  END IF;

  COMMIT;
  ret_val := 0;
<<end_proc>> NULL;
END;
/
show errors;

/***************************************************************************/
/* Permission Granting Statements.                                         */
/*                                                                         */
/* Copyright (C) 1999. First Choice Software. All Rights Reserved.         */
/***************************************************************************/

grant execute on part_transfer to clarify_user;
begin
  dbms_sql.parse(dbms_sql.open_cursor, 'drop public synonym part_transfer', dbms_sql.native);
  exception
  when others
    then null;
end;
/
create public synonym part_transfer for part_transfer;

grant execute on trans_internal to clarify_user;
begin
  dbms_sql.parse(dbms_sql.open_cursor, 'drop public synonym trans_internal', dbms_sql.native);
  exception
  when others
    then null;
end;
/
create public synonym trans_internal for trans_internal;

/***************************************************************************/
/* Test Driver.                                                            */
/*                                                                         */
/* Copyright (C) 1999. First Choice Software, Inc. All Rights Reserved     */
/***************************************************************************/

CREATE OR REPLACE PROCEDURE test_proc
IS
  ret_val NUMBER;
BEGIN
  part_transfer('EXPGL', '', 1, 'San Jose', 'Bin A', 1, 'Packing Container', '', 'Quantity',
                2, '', 'sa', 'Ref ID', 'Notes', '', 'Wo num', 2, 3.44, 1, ret_val);
  dbms_output.put_line(ret_val);
END test_proc;
/
show errors;

grant execute on test_proc to clarify_user;
begin
  dbms_sql.parse(dbms_sql.open_cursor, 'drop public synonym test_proc', dbms_sql.native);
  exception
  when others
    then null;
end;
/
create public synonym test_proc for test_proc;

commit;