/***************************************************************************/
/***  Name         : log_help.sql                                        ***/
/***                                                                     ***/
/***  Description  : Stored procedures used as helpers for logistics     ***/
/***                  modules.                                           ***/
/***                                                                     ***/
/***  Author       : First Choice Software, Inc                          ***/
/***                 4412 Spicewood Springs Road Suite 701               ***/
/***                 Austin, TX 78759                                    ***/
/***                                                                     ***/
/***  Usage        : First Choice Confidential                           ***/
/***                                                                     ***/
/***  Preconditions: None known                                          ***/
/***                                                                     ***/
/***  Postcondition: None known                                          ***/
/***                                                                     ***/
/***  Platforms    : This version supports SQL Server version 6.0 or     ***/
/***                  later.                                             ***/
/***                                                                     ***/
/***                                                                     ***/
/***  Copyright (C) 1997 First Choice Software, Inc.                     ***/
/***  All Rights Reserved                                                ***/
/***                                                                     ***/
/***************************************************************************/

/***************************************************************************/
/***     Header declarations.                                            ***/
/***************************************************************************/

use <db>
go

					/* Drop stored procedures, if      */
					/*  if defined                     */
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'get_list_item')
   DROP PROCEDURE get_list_item
go

/***************************************************************************/
/***     Get item from a Clarify List (gbst_lst).                        ***/
/***************************************************************************/

CREATE PROCEDURE get_list_item @list_name VARCHAR(80),
                               @list_item VARCHAR(80),
                               @ret_item VARCHAR(80) OUTPUT,
                               @ret_objid INT OUTPUT AS
BEGIN
  DECLARE @list_objid INT

                                        /* Find the list. If not found,    */
                                        /*  error and exit.                */
  SELECT @list_objid = objid FROM table_gbst_lst WHERE title = @list_name
  IF @list_objid IS NULL
  BEGIN
     SELECT @ret_item = ''
     SELECT @ret_objid = 0
     GOTO end_proc
  END

                                        /* If no item given, find the      */
                                        /*  default. Else make sure that   */
                                        /*  the item is there (and active).*/
  IF @list_item = ""
  BEGIN
     SELECT @ret_item = title, @ret_objid = objid FROM table_gbst_elm 
       WHERE gbst_elm2gbst_lst = @list_objid AND state = 2
  END
  ELSE
  BEGIN
     SELECT @ret_item = title, @ret_objid = objid FROM table_gbst_elm
       WHERE gbst_elm2gbst_lst = @list_objid AND title = @list_item AND
             (state = 0 OR state = 2)
     IF @ret_item IS NULL
     BEGIN
        SELECT @ret_item = ''
        SELECT @ret_objid = 0
        GOTO end_proc
     END
  END
end_proc:
END
go

/***************************************************************************/
/***     Permission granting statements.                                 ***/
/***************************************************************************/

GRANT EXECUTE ON get_list_item TO PUBLIC
go

quit